package com.ruoyi.music.service.impl;

import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.music.domain.MusicFile;
import com.ruoyi.music.mapper.MusicFileMapper;
import com.ruoyi.music.service.IMusicService;
import jcifs.smb.SmbFile;
import jcifs.smb.SmbFileInputStream;
import org.jaudiotagger.audio.AudioFile;
import org.jaudiotagger.audio.AudioFileIO;
import org.jaudiotagger.tag.FieldKey;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Date;
import java.util.List;

@Service
public class MusicServiceImpl implements IMusicService {

    @Autowired
    private MusicFileMapper musicFileMapper;

    @Override
    public void scanSmbPath(String smbPath, String username, String password) {
        try {
            String authStr = username + ":" + password + "@";
            String fullPath = "smb://" + authStr + smbPath.replace("\\", "/");

            SmbFile rootFile = new SmbFile(fullPath);
            rootFile.connect();

            processDirectory(rootFile, SecurityUtils.getUserId(), SecurityUtils.getDeptId());
        } catch (Exception e) {
            throw new RuntimeException("SMB连接失败：" + e.getMessage());
        }
    }

    private void processDirectory(SmbFile dir, Long userId, Long deptId) throws Exception {
        for (SmbFile file : dir.listFiles()) {
            if (file.isDirectory()) {
                processDirectory(file, userId, deptId);
            } else {
                if (file.getName().toLowerCase().matches(".*\\.(mp3|flac|wav)$")) {
                    saveMusicInfo(file, userId, deptId);
                }
            }
        }
    }

    private void saveMusicInfo(SmbFile file, Long userId, Long deptId) {
        File tempFile = null;


            try {
                // 创建临时文件
                tempFile = File.createTempFile("music", null);
                try (InputStream in = new SmbFileInputStream(file);
                     FileOutputStream out = new FileOutputStream(tempFile)) {
                    byte[] buffer = new byte[4096];
                    int bytesRead;
                    while ((bytesRead = in.read(buffer)) != -1) {
                        out.write(buffer, 0, bytesRead);
                    }
                } catch (Exception e) {
                System.err.println("解析失败：" +   e.getMessage());
            }
            AudioFile audioFile = AudioFileIO.read(tempFile);
            MusicFile music = new MusicFile();
            music.setFilePath(file.getCanonicalPath());
            music.setFileName(file.getName());
            music.setTitle(audioFile.getTag().getFirst(FieldKey.TITLE));
            music.setArtist(audioFile.getTag().getFirst(FieldKey.ARTIST));
            music.setAlbum(audioFile.getTag().getFirst(FieldKey.ALBUM));
            music.setDuration(audioFile.getAudioHeader().getTrackLength());
            music.setFileSize(file.length());
            music.setUserId(userId);
            music.setDeptId(deptId);
            music.setCreateTime(new Date());
            musicFileMapper.insertMusicFile(music);
        } catch (Exception e) {
            System.err.println("解析失败：" + file.getName() + " - " + e.getMessage());
        }
    }

    @Override
    public List<MusicFile> selectMusicList(MusicFile musicFile) {
        return musicFileMapper.selectMusicFileList(musicFile);
    }
}